!***********************************************************************
!*****************************  KEK< High Energy Accelerator Research  *
!*****************************  Organization                           *
!***** u c d e t e c c *******                                         *
!*****************************     EGS5.0 USER CODE - 26 Sep 2014/1600 *
!   Close pict file at each times after preset history.                *
!   Clear random number after new energy.                              *
!***********************************************************************
!***********************************************************************
!                                                                      *
!  PROGRAMMERS:  H. Hirayama                                           *
!                Radiation Science Center                              *
!                Applied Science Laboratory                            *
!                KEK, High Energy Accelerator Research Organization    *
!                1-1, Oho, Tsukuba, Ibaraki, 305-0801                  *
!                Japan                                                 *
!                                                                      *
!                E-mail:      hideo.hirayama@kek.jp                    *
!                Telephone:   +81-29-864-5489                          *
!                Fax:         +81-29-864-1993                          *
!                                                                      *
!                Y. Namito                                             *
!                Radiation Science Center                              *
!                Applied Research Laboratory                           *
!                KEK, High Energy Accelerator Research Organization    *
!                1-1, Oho, Tsukuba, Ibaraki, 305-0801                  *
!                Japan                                                 *
!                                                                      *
!                E-mail:      yoshihito.namito@kek.jp                  *
!                Telephone:   +81-29-864-5489                          *
!                Fax:         +81-29-864-1993                          *
!                                                                      *
!***********************************************************************
!***********************************************************************
! The following shows the geometry for ucnai.f.                        *
! This user code corresponds to ucnai3.mor for egs4.                   *
! Use Ranlux random number generator.                                  *
!***********************************************************************
!                                                                      *
!             --------------------------------------                   *
!             Radial-Z Geometry (ucdetecc example)                     *
!             --------------------------------------                   *
!                                                                      *
!                                                                      *
!                Y (X into page)                                       *
!                ^                                                     *
!                |                                                     *
!                |                                                     *
!                +----+-------------+----+----- 4.81 cm                *
!                | Air| Air         | Air|                             *
!                +----+-------------+----+----- 3.81 cm                *
!                |    |             |    |                             *
!                |    | Nai         |    |                             *
!                |    |  or         |    |                             *
!                | Air| Ge          | Air|                             *
!      1.253 MeV |    |             |    |                             *
!      ==========>+---+-------------+----+----------> Z                *
!     photons    0   2.0       2.0+zthick 4.0+zthick                   *
!                                                                      *
!***********************************************************************
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12

!-----------------------------------------------------------------------
!------------------------------- main code -----------------------------
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! Step 1: Initialization
!-----------------------------------------------------------------------

      implicit none

!     ------------
!     EGS5 COMMONs
!     ------------
      include 'include/egs5_h.f'                ! Main EGS "header" file

      include 'include/egs5_bounds.f'
      include 'include/egs5_brempr.f'
      include 'include/egs5_edge.f'
      include 'include/egs5_media.f'
      include 'include/egs5_misc.f'
      include 'include/egs5_thresh.f'
      include 'include/egs5_uphiot.f'
      include 'include/egs5_useful.f'
      include 'include/egs5_usersc.f'
      include 'include/egs5_userxt.f'
      include 'include/randomm.f'

!     ----------------------
!     Auxiliary-code COMMONs
!     ----------------------
      include 'auxcommons/aux_h.f'   ! Auxiliary-code "header" file

      include 'auxcommons/cyldta.f'
      include 'auxcommons/edata.f'
      include 'auxcommons/etaly1.f'
      include 'auxcommons/georz.f'
      include 'auxcommons/geoxyz.f'
      include 'auxcommons/instuf.f'
      include 'auxcommons/nfac.f'
      include 'auxcommons/lines.f'
      include 'auxcommons/pladta.f'
      include 'auxcommons/watch.f'

      common/totals/depe                             ! Variables to score
      real*8 depe

!**** integer                                                ! Arguments
      integer ins
      
      real*8                                           ! Local variables
     * availke,avph,avpe,avte,ct,ct0,deltae,phai,rair,rdet,rphai,sigph,
     * small,pef,tef,sigpe,sigte,ekin,etot,totke,wtin,wtsum,zair,zthick,
     * depesum,rnnow

      real*8 ph(50)
     
      real                                             ! Local variables
     * eav,elow,eup

      integer
     * i,icases,idin,ie,iflxr,ihist,ii,iii,iplsrc,ismat,isosrc,
     * j,k,mode,n

      character*24 medarr(4)

!     ====================
      call counters_out(0)
!     ====================

      mode=1
!          0: include pegs
!          1: use material data created by mode=0

      npreci=3    ! 0:precision of x-, y-, z-position in 4 digits   PICT
                  ! 1:precision of x-, y-, z-position in 8 digits   PICT
                  ! 0,1: for EGS4WIN32
                  ! 2,3: for CGview 
                  ! 2: fixed format
                  ! 3: free format

!-----------------------------------------------------------------------
! Step 2: pegs5-call
!-----------------------------------------------------------------------
!     ==============
      call block_set                 ! Initialize some general variables
!     ==============

!     ---------------------------------
!     define media before calling PEGS5
!     ---------------------------------

      nmed=4
      medarr(1)='GE                      '
      medarr(2)='NAI                     '
      medarr(3)='BGO                     '
      medarr(4)='AIR AT NTP              '

      do j=1,nmed
        do i=1,24
          media(i,j)=medarr(j)(i:i)
        end do
      end do  

      chard(1) = 1.0d0       !  automatic step-size control
      chard(2) = 1.0d0
      chard(3) = 1.0d0
      chard(4) = 1.0d0

!     -----------------------------------
!     Run KEK PEGS5 before calling HATCH
!     -----------------------------------
      if(mode.eq.0) then
        write(6,'(A/)') ' PEGS5-call comes next'

!       ==========
        call pegs5
!       ==========
      end if

!-----------------------------------------------------------------------
! Step 3: Pre-hatch-call-initialization
!-----------------------------------------------------------------------
!     nreg : number of region
      
      nplan=4                  !Number of planes
      ncyl=2                   !Number of cylinders

      nreg=(nplan-1)*ncyl+3    !Number of regions (including outside
                               !vacuum region)
      irz=nreg-3

! Absorbed energy spectrum format
      write(6,'(A/A)')
     *   ' Key in absorbed energy spectrum format',
     *   ' 0: Points, 1: Histogram'
      read(5,*) ihist

      write(6,'(A)') ' Key in Material number.  1:Ge, 2:NaI, 3:BGO'
      read(5,*) ismat

      write(6,'(A/A)')
     *   ' Do you want to produce Fluorescent X-ray of detector ?',
     *   '      (0:no, 1:yes)'
      read(5,*) iflxr

!   Set medium index for each region
!     Vacuum region
      med(1)=0
      med(8)=0
      med(9)=0

!     Air region
      med(2)=4
      med(3)=4
      med(5)=4
      med(6)=4
      med(7)=4

!     Detector region
      med(4)=ismat
      iedgfl(4)=iflxr

      write(6,'(A/A)') ' Key in Particle type',
     *   ' (-1:electron, 0:photon, 1:positron)'
      read(5,*) iqin 

!-----------------------------------------------------------------------
! Step 4:   hatch-call : Ordinary set at Step 5
!-----------------------------------------------------------------------
!     --------------------------------------------------------
!     Random number seeds.  Must be defined before call hatch
!     or defaults will be used.  inseed (1- 2^31)
!     --------------------------------------------------------
      luxlev = 1
      inseed=1

!     =============
      call rluxinit  ! Initialize the Ranlux random-number generator
!     =============

      emaxe = 0.d0

      write(6,'(A/A)')
     *   'Start ucdetecc ','Call hatch to get cross-section data'

!     ------------------------------
!     Open files (before HATCH call)
!     ------------------------------
      if(mode.eq.0) then
        open(UNIT=KMPI,FILE='pgs5job.pegs5dat',STATUS='old')
      else
        open(UNIT=KMPI,FILE='detecc.dat',STATUS='old')
      end if
      open(UNIT=KMPO,FILE='egs5job.dummy',STATUS='unknown')

      write(6,'(/A/)') ' HATCH-call comes next'

!     ==========
      call hatch
!     ==========

!     ------------------------------
!     Close files (after HATCH call)
!     ------------------------------
      close(UNIT=KMPI)
      close(UNIT=KMPO)
! ----------------------------------------------------------
! Print various data associated with each media (not region)
! ----------------------------------------------------------
      write(6,'(/A)') ' Quantities associated with each MEDIA:'
      do j=1,nmed
        write(6,'(/1X,24A1)') (media(i,j),i=1,24)
        write(6,'(5X,A,G15.7,A,G15.7,A)')
     *     ' rho=',rhom(j),' g/cu.cm     rlc=',rlcm(j),' cm'
        write(6,'(5X,A,G15.7,A,G15.7,A)') 
     *     ' ae=',ae(j),' MeV    ue=',ue(j),' MeV'
        write(6,'(5X,A,G15.7,A,G15.7,A/)') 
     *     ' ap=',ap(j),' MeV    up=',up(j),' MeV'
      end do

1100  write(6,'(A/A)') ' Key in Particle Kinetic Energy in MeV',
     *   ' 0.0 means end of run'
      read(5,*) ekein

      if(ekein.le.0.d0) go to 9999

!     --------------------------------------------------------
!     Random number seeds.  Must be defined before call hatch
!     or defaults will be used.  inseed (1- 2^31)
!     --------------------------------------------------------
      luxlev = 1
      inseed=1

      kount=0  ! must be set to produce same random number
      mkount=0

!     =============
      call rluxinit  ! Initialize the Ranlux random-number generator
!     =============

!-----------------------------------------------------------------------
! Step 5:  Determination-of-incident-particle-parameters
!          Ordinary set at Step 4
!-----------------------------------------------------------------------
      etot=ekein
      if(iqin.ne.0) etot=ekein+RM

      if(etot.gt.ue(ismat)) then
        write(6,'(A,G10.3,A)') 
     *     ' Total energy must be less or eqal to ', ue(ismat),'MeV'
        go to 1100
      end if
      if(ekein.gt.up(ismat)) then
        write(6,'(A,G10.3,A)') 
     *     ' Kinetic energy must be less or eqal to ', up(ismat),'MeV'
        go to 1100
      end if

! Define initial variables for incident particle normally incident
! on the slab
      xin=0.0            ! Incident at origin
      yin=0.0
      zin=0.0
      uin=0.0            ! Moving along z axis
      vin=0.0
      win=1.0
      irin=2             ! Starts in region 2, could be 1
      wtin=1.0           ! Weight = 1 since no variance reduction used

!-----------------------------------------------------------------------
! Step 6:  Initialization-for-howfar
!-----------------------------------------------------------------------
      zair=2.0   ! Air thickness before and after slab in cm

      write(6,*) ' Key in detector thickness in cm'
      read(5,*) zthick    ! Detector thickness in centimeters

! Parameter to define graphic size. It is better that the   **PICT**
! width of each axis is nearly same.                        **PICT**
      xmin=-6.0
      xmax=6.0
      ymin=-6.0
      ymax=6.0
      zmin=-1.0 
      zmax=zthick+2.0*zair+2.0              ! PICT

      nplan=4
      do i=1,nplan
        do j=1,3
          if(j.ne.3) then
            pnorm(j,i)=0.d0
          else
            pnorm(j,i)=1.d0
          end if
          pcoord(j,i)=0.d0
        end do
      end do

      pcoord(3,2)=pcoord(3,1)+zair
      pcoord(3,3)=pcoord(3,2)+zthick
      pcoord(3,4)=pcoord(3,3)+zair

      rdet=3.81      ! Radius of detector in cm
      rair=1.0       ! Air region around detector

      ncyl=2
      cyrad(1)=rdet
      cyrad(2)=rdet+rair

      do i=1,ncyl
        cyrad2(i)=cyrad(i)*cyrad(i)
      end do

! Definition of planes

! Set zbound to zero to begin with
      zbound(1)=0.0

! Now put in the exceptions

      zbound(2)=pcoord(3,2)
      zbound(3)=pcoord(3,3)
      zbound(4)=pcoord(3,4)

1200  write(6,'(A)') ' Key in number of cases (0 means new energy)'
      read(5,*) ncases
      
      if(ncases.eq.0) go to 1100

      open(39,file='egs5job.pic',status='unknown')     ! PICT

!  Parameter to define graphic size. It is better that the   **PICT**
!  width of each axis is nearly same.                        **PICT**
!     --------------------------------------------
!     Write plane information for geometry drawing
!     --------------------------------------------
      if(npreci.eq.1) then 
!     ------------------------
!     Set parameter for PICT32
!     ------------------------
        call geomout(1,nplan)

        fnorm=dmax1(xmax-xmin+2,ymax-ymin+2,zmax-zmin)   ! PICT
        write(39,'(7E10.3)') xmin,xmax,ymin,ymax,xmin,zmax,fnorm  ! PICT 
 
      else
!     ------------------------
!     Set parameter for CGVIEW
!     ------------------------
        if(npreci.eq.2) write(39,fmt="('GSTA-TIME')")
        if(npreci.eq.3) write(39,fmt="('GSTA-FREE-TIME')")
        write(39,fmt="('CYLS')")
        write(39,fmt="(2I6)") ncyl,nplan
        write(39,fmt="(6E11.3)") (cyrad(i),i=1,ncyl)
        write(39,fmt="(6E11.3)") (zbound(i),i=1,nplan)
        write(39,fmt="('GEND')")

        write(39,fmt="('MSTA')")
        write(39,fmt="(i4)") nreg
        write(39,fmt="(15I4)") (med(i),i=1,nreg)
        write(39,fmt="('MEND')")
      end if

!-----------------------------------------------------------------------
! Step 7:  Initialization-for-ausgab
!-----------------------------------------------------------------------

      ncount = 0
      ilines = 0
      nwrite = 10
      nlines = 10
      idin = -1
      totke = 0.
      wtsum = 0.

      iii=0

      isosrc=0    !  0:monodirectional, 1:point isotropic.

      iplsrc=1     !  0:beam, 1:plane source.

!     Energy bin width 
      deltae=ekein / 50

!     Zero the variables
      depe=0.D0
      depesum=0.D0
      pef=0.D0
      tef=0.D0
      do j=1,50
        ph(j)=0.D0
      end do

!-----------------------------------------------------------------------
! Step 8:  Shower-call
!-----------------------------------------------------------------------
                                             ! -------------------------
      do i=1,ncases                          ! Start of call shower loop
                                             ! -------------------------
        ct0=5.0/sqrt(5.0*5.0+5.0*5.0)
        iii=iii+1

        write(39,'(A,I5)') '0',iii    ! PICT

!       ----------------------
!       Select incident energy
!       ----------------------
        ekin = ekein
        wtin = 1.0

        wtsum = wtsum + wtin               ! Keep running sum of weights
        etot = ekin + iabs(iqin)*RM        ! Incident total energy (MeV)
        availke = etot + iqin*RM        ! Available K.E. (MeV) in system
        totke = totke + availke                 ! Keep running sum of KE

        if (isosrc.eq.1) then             ! Sample isotropically.
          call randomset(ct)
          win = 1.D0 - ct*(1.0-ct0)
          call randomset(rphai)
          phai=3.141569*2.0*rphai
          uin = dsqrt(1.D0 - win*win)*dcos(phai)
          vin = dsqrt(1.D0 - win*win)*dsin(phai)
        end if

        if(iplsrc.eq.1) then   ! Plane Source
          call randomset(ct)
          yin=(2.0*ct-1.0)*cyrad(1)
        end if

        if (ncount .le. nwrite .and. ilines .le. nlines) then
          ilines = ilines + 1
        end if

!       =========================================================
        call shower (iqin,etot,xin,yin,zin,uin,vin,win,irin,wtin)
!      ========================================================

!      If some energy is deposited inside detector add pulse-height
!      and efficiency.

        if (depe .gt. 0.D0) then
          ie=depe/deltae + 1
          if (ie .gt. 50)  ie = 50
          ph(ie)=ph(ie)+wtin
          tef=tef + wtin
          if(depe .ge. ekein*0.999) pef=pef +wtin
          depesum=depesum+depe
          depe = 0.D0
        end if

        ncount = ncount + 1         ! Count total number of actual cases

                                               ! -----------------------
      end do                                 ! End of CALL SHOWER loop
                                               ! -----------------------

      write(39,'(A)') '9'                 ! PICT

      call plotxyz(99,0,0,0.D0,0.D0,0.D0,0.D0,0,0.D0,0.D0)     ! PICT

      close(UNIT=39)

!-----------------------------------------------------------------------
! Step 9:  Output-of-results
!-----------------------------------------------------------------------

      write(6,'(/A)') ' Pulse height distribution '
      write(6,'(A)') ' E (upper-edge MeV )    Counts/MeV/incident  Gosa'
      open(17,FILE='egs5job.17o')
      small=1.0d-10
!     --------------------------
!     Pulse height distribution
!     --------------------------
      write(6,'(/A)') ' Pulse height distribution '
      do ie=1,50
        elow=deltae*(ie-1)
        eup=deltae*ie
        eav=(elow+eup)*0.5
        if (elow .gt. ekein ) go to 440
        
        avph = ph(ie)/ncases
        sigph=dsqrt(ph(ie))/ncases
        write(6,'(A,G10.4,A,G15.5,A,G15.6,A)') 
     *     ' E (upper-edge --',eup,' MeV )=',avph,'+-',sigph,
     *     ' counts/incident'
        if(ihist.eq.0) then
          write(17,'(1P3G10.3)') eav,avph,sigph
        else
          write(17,'(1P3G10.3)') elow,avph,sigph
          write(17,'(1P3G10.3)') eup,avph,sigph
        end if
      end do
 
440   continue
      if(ihist.eq.1) write(17,'(1P3G10.3)') eup,small,0.5*small
      close(UNIT=17)
      
!     -----------------------------------
!     Calculate average and its deviation
!     -----------------------------------

!     ---------------
!     Peak efficiency
!     ---------------
      avpe = pef/ncases*100.0
      sigpe = sqrt(pef)/ncases*100.0
      write(6,'(A,G15.5,A,G15.5,A)') 
     *   ' Peak efficiency =',avpe,'+-',sigpe,' %'


!     ----------------
!     Total efficiency
!     ----------------
      avte = tef/ncases*100.0
      sigte = dsqrt(tef)/ncases*100.0
      write(6,'(A,G15.5,A,G15.5,A)') 
     *   ' Total efficiency =',avte,'+-',sigte,' %'
      write(6,'(A,G10.3,A)') 
     *   ' Energy deposition =',depesum,' MeV'

      go to 1200

9999  stop

      end

!-------------------------last line of main code------------------------

!-------------------------------ausgab.f--------------------------------
! Version:   030831-1300
! Reference: SLAC-265 (p.19-20, Appendix 2)
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12

! ----------------------------------------------------------------------
! Required subroutine for use with the EGS5 Code System
! ----------------------------------------------------------------------
! A simple AUSGAB to:
!
!   1) Score energy deposition
!   2) Print out stack information
!   3) Print out particle transport information (if switch is turned on)
!

! ----------------------------------------------------------------------

      subroutine ausgab(iarg)

      implicit none

      include 'include/egs5_h.f'                ! Main EGS "header" file

      include 'include/egs5_epcont.f'    ! COMMONs required by EGS5 code
      include 'include/egs5_misc.f'
      include 'include/egs5_stack.f'
      include 'include/egs5_useful.f'

      include 'auxcommons/aux_h.f'   ! Auxiliary-code "header" file

      include 'auxcommons/etaly1.f'        ! Auxiliary-code COMMONs
      include 'auxcommons/geortz.f'
      include 'auxcommons/instuf.f'
      include 'auxcommons/lines.f'
      include 'auxcommons/ntaly1.f'
      include 'auxcommons/watch.f'

      common/totals/depe                         ! Variables to score
      real*8 depe

      integer                                                ! Arguments
     * iarg

      real*8                                           ! Local variables
     * edepwt
     
      integer
     * ie,iql,irl

!     ------------------------
!     Set some local variables
!     ------------------------
      irl = ir(np)
      iql = iq(np)
      edepwt = edep*wt(np)

!     ------------------------------------
!     Output particle information for plot
!     ------------------------------------
      if(ncases.le.100)  then
        call plotxyz(iarg,np,iq(np),x(np),y(np),z(np),e(np),ir(np),
     *     wt(np),time(np))
      end if

!     ----------------------------------------------
!     Score energy deposition inside NaI detector
!     ----------------------------------------------
      if (irl.eq.4) then
        depe = depe + edepwt
      end if

      return

      end

!--------------------------last line of ausgab.f------------------------

!-------------------------------howfar.f--------------------------------
! Version:   040810-1300
! Reference: SLAC-265 (p.19-20, Appendix 2)
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12
! ----------------------------------------------------------------------
! Required (geometry) subroutine for use with the EGS5 Code System
! ----------------------------------------------------------------------
! This is a general-purpose, R-Z HOWFAR. 
! ----------------------------------------------------------------------

      subroutine howfar

      implicit none

      include 'include/egs5_h.f'                ! Main EGS "header" file

      include 'include/egs5_epcont.f'    ! COMMONs required by EGS5 code
      include 'include/egs5_stack.f'

      include 'auxcommons/aux_h.f'   ! Auxiliary-code "header" file

      include 'auxcommons/cyldta.f'        ! Auxiliary-code COMMONs
      include 'auxcommons/georz.f'
      include 'auxcommons/geoxyz.f'
      include 'auxcommons/instuf.f'
      include 'auxcommons/pladta.f'

      real*8                                           ! Local variables
     * dist,tcyl
      integer
     * ihit,ipl1,ipl2,irl,irnxt1,irnxt2,nannu,ncl1,ncl2,nslab

      irl = ir(np)

      if (irl .le. 0) then
        write(6,*) 'Stopped in howfar with irl <= 1'
        stop
      end if

      if (irl .eq. 1. or. irl .ge. irz+2) then
        idisc = 1  ! ---------------------------------------------------
        return     ! Particle outside geometry - return to ELECTR/PHOTON
      end if       ! ---------------------------------------------------

!     ----------------------------------
!     Get slab number and annulus number
!     ----------------------------------
      nslab = (irl -2 ) / ncyl +1                      ! Slab number
      nannu = irl -1 - ncyl * (nslab -1)               ! Annulus number
!     --------------------
!     Check in Z-direction
!     --------------------
      ipl1 = nslab + 1
      ipl2 = nslab

      if (nslab .lt. nplan-1) then
        irnxt1 = irl + ncyl
      else
        irnxt1 = irz + 2
      end if
      
      if (nslab .gt. 1 ) then
        irnxt2 = irl - ncyl
      else 
        irnxt2 = 1
      end if

      if (w(np) .gt. 0.0) then
        dist = (zbound(ipl1)-z(np))/w(np)
        if (dist .lt. ustep) then
          ustep=dist
          irnew=irnxt1
        end if
      else if (w(np) .lt. 0.0) then
        dist = -(z(np) - zbound(ipl2))/w(np)
        if (dist .lt. ustep) then
          ustep = dist
          irnew=irnxt2
        end if
      end if

!     --------------------
!     Check in R-direction
!     --------------------

      if (nannu .lt. ncyl) then
        irnxt2 = irl +1
      else 
        irnxt2 = irz + 3
      end if
     
      if (nannu .gt. 1) then
        irnxt1 = irl -1
        ncl2 = nannu
        ncl1 = nannu -1
        call cyl2 (ncl1, irnxt1, ncl2, irnxt2)
      else                             ! Inner-most cylinder---special case
        call cylndr(1,1,ihit,tcyl)
        if (ihit .eq. 1) then
          call chgtr(tcyl,irnxt2)
        end if
      end if

                                               ! -----------------------
      return                                   ! Return to ELECTR/PHOTON
                                               ! -----------------------
      end

!--------------------------last line of howfar.f------------------------
!-------------------------------photon_vr.f--------------------------------
! Version:   071121-1000
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12
! ----------------------------------------------------------------------
! Required subroutine for forced collsion at region 2 once only.
! Apply macro developped by Alex.
! ----------------------------------------------------------------------

      subroutine photon_vr(dpmfp,gmfp)

      implicit none

      real*8 rnnow,dpmfp,gmfp                                ! Arguments

      return
      end
!--------------------------last line of photon_vr.f------------------------
